package Netflow::Collector::Exception;
use v5.10;
use Moose;
extends 'Throwable::Error';
use namespace::autoclean;
no Moose;
1;
__END__

=head1 NAME

Netflow::Collector::Exception extends Throwable::Error

=cut


